function compbode(arg1,arg2,arg3,arg4)
%COMPBODE Lag and lead compensator design with the Bode diagrams (TFI).
%    Possible uses:
%    COMPBODE,gp,gc - from Transfer Function Interpreter
%    COMPBODE('gp','gc') - from Matlab
%    gp: transfer function of the plant (input)
%    gc: transfer function of the lead compensator (output)
%    If COMPBODE is called without arguments, transfer functions gp and gc
%    can be specified in interactive mode.

% TFI Environment (A.Civolani & G.Marro) - 1994-96.

global ABIL TRF1 TRF2 stknum1 stkden1 stkkc stktaui stktaud repco TT TT1...
 stkn1 stkd1 stkn2 HALEG FIGLEG LABLEG SUBPLEG gi pbs SC

nfig=gcf; nfigs=int2str(nfig); ABIL=0; flgpp=0; FIGLO=resfiglo;
FIGLEG=nfig; HALEG=[]; LABLEG=[]; SUBPLEG=0; pbs=zeros(1,20);
wfont='normal'; % wfont='bold';
framesc=get(0,'DefaultFigureColor');
ms1='\n **** error: you cannot change figure while running\n';
if exist('legen#.mat'), load legen#, else, legen=0; islegen=0; end

if nargin==0
 arg1='xx';
 mess='   enter the transfer function of the plant (input) : ';
 gi=input(mess,'s');
 if gcf~=nfig, fprintf(ms1), return, end
 if isempty(gi), return, end
 if gi=='0', return, end
 if ~exist([gi,'.mat'])
  disp([' **** error: ',gi,'.mat does not exist']), return
 end
 mess='   enter the transfer function of the compensator (output) : ';
 gj=input(mess,'s');
 if gcf~=nfig, fprintf(ms1), return, end
 if isempty(gj), return, end
 if gj=='0', return, end
elseif nargin>=2
 if ~exist([arg1,'.mat'])
  disp([' **** error: ',arg1,'.mat does not exist']), return
 else
  gi=arg1; gj=arg2;
 end
else
 disp(' **** error: not enough input arguments'), return
end
[z,p,k,str1]=exportf(gi,1); if isempty(k), return, end
if str1=='z'
 disp('   for discrete-time systems it is necessary to convert to w-plane')
end
[num,den]=dezpk(z,p,k);
tfblocks(3,gj,gi,'','','','','','','','')
%
[vcol1,vcol2,repco,repcos]=coltbl; colbg=vcol1(8,:); 
gg=coltbl('b'); or=coltbl('o'); col='g'; colc=vcol1(1,:);
%
tfm(z,p,k,gi)
%
set(0,'DefaultUIControlUnits','normalized')
nfig=gcf; ordf(3), clf, set(gcf,'Color',colbg)
VFULL=get(gcf,'Position'); PO=get(gca,'Position');
SC=diag([1/VFULL(3);1/VFULL(4);1/VFULL(3);1/VFULL(4)]);
nfig1=nfig+1; figure(nfig1), ordf(7), clf, set(gcf,'Color',colbg)
figure(nfig)

[gain,phi,omega]=frespc(z,p,k,3.5,'o');

TT=[omega(1), omega(length(omega))];
stknum1=z; stkden1=p; stkkc=k; 
ccregaux(25)
if islegen
 LABLEG=['''',gi,''''];
 if legen, eval(['legend(HALEG,',LABLEG,')']), end
end   
figure(nfig), ordf(3)
Pos=[10 5 500 15]*SC;
xyx=input(['   **** figure ',nfigs,' **** '],'s');
set(pbs(17),'Enable','off')
set(pbs(18),'Enable','off')
BB=VFULL(4);
Pos=[150 BB-120 200 20]*SC;
pbs(20)=uicontrol(gcf,...
'Style','Text',...
'Position',Pos,...
'BackgroundColor',colbg,...
'ForegroundColor',vcol2(2,:),...
'Visible','off',...
'String','ILLEGAL VALUE');
gain=stkn1; phi=stkd1; omega=stkn2;
lomega=length(omega); 
%Find the cross frequency
n=length(gain);
gaindb=20*log10(gain);
i=1;
while gaindb(i)>0,
   istar=i;
   i=i+1;
end

omegastar=omega(istar); marginefase=180+phi(istar); 

%Evaluate the boundary curves
gaindx=gain(istar+1:n);
gainsx=gain(1:istar);
phidx=phi(istar+1:n);
phisx=phi(1:istar);

Mdx=1./gaindx;
Msx=gainsx;
phimaxdx=(180/pi)*acos(1./Mdx);
phimaxsx=(180/pi)*acos(1./Msx);
phibounddx=phidx+phimaxdx;
phiboundsx=phisx-phimaxsx;
phibound=[phiboundsx;phibounddx];

subplot(2,1,1), VV1=axis; hold on
semilogx([omegastar omegastar], [VV1(3) 0],'linewidth',.5,'linestyle','-.','color',gg), axis(VV1)
%hold off

subplot(2,1,2), VV2=axis; hold on
semilogx([omegastar omegastar], [VV2(3) VV2(4)],'linewidth',.5,'linestyle','-.','color',gg), axis(VV2)

figure(gcf),
mess1=sprintf('crossover angular frequency : %.4g rad/sec, ',omegastar);
mess2=sprintf('phase margin : %.4g degrees',marginefase);
mess=[mess1,mess2];
set(pbs(19),'String',mess,'ForegroundColor',vcol2(6,:),'Visible','on')

xyx=input(['   **** figure ',nfigs,' **** '],'s');

%Input phase margin

Mphi=[];
while isempty(Mphi)
  Mphi=input('   enter the phase margin for design : ');
end
ph=-180+Mphi; 
figure(nfig)

semilogx([VV2(1) VV2(2)], [ph ph],'k','linewidth',.5,'linestyle','-.','color',gg)
semilogx(omega, phibound,'k','linewidth',.5,'linestyle','-.','color',gg)
hold off

mess=sprintf('choose the angular frequency on the figure with mouse, press return to continue');
set(pbs(19),'String',mess,'ForegroundColor',vcol2(9,:),'Visible','on')

stkzr=[]; stkpr=[]; stkkr=[]; stkalpha=[]; stktau=[]; stktype1=[];
nnrip=1; xyz=[]; b=1;
while (isempty(xyz))&(b==1) %%%% loop to choose omega_0
xx=[];
while isempty(xx)
  [xx,yy,b]=ginput(1); if isempty(b), b=3; end, if b==3, xx=1; xyz=1; end
  if b==1
    set(pbs(20),'Visible','off')
    if ~isempty(xx)
      xx1=xx*ones(1,lomega);
      [kk,ii]=min(abs(omega-xx1));
      omegad=omega(ii);
      gaind=gain(ii); gainddb=20*log10(gaind);
      phiboundd=phibound(ii);
      phid=phi(ii);
      if xx<omegastar
        type1=1; % lag
        if (gainddb<0)|((phid-ph)<0)|(phiboundd>ph)
          xx=[]; set(pbs(20),'Visible','on') 
        end
      else
        type1=2; % lead
        if (gainddb>0)|(phiboundd<ph)
          xx=[]; set(pbs(20),'Visible','on') 
        end
      end
   end
 end % if b==1
end

if b==1 
  stktype1=[stktype1;type1];
  xx1=xx*ones(1,lomega);
  [kk,ii]=min(abs(omega-xx1));
  omegad=omega(ii);
  gaind=gain(ii); gainddb=20*log10(gaind);
  phid=phi(ii);
  colc=vcol1(nnrip,:);

  subplot(2,1,1), hold on
  semilogx([omegad omegad], [0 gainddb],'linewidth',1,'linestyle','-','color',colc), axis(VV1)
  %hold off

  subplot(2,1,2), hold on
  semilogx([omegad omegad], [ph phid],'linewidth',1,'linestyle','-','color',colc), axis(VV2)
  %hold off

  M=gaind;
  P=(phid-ph)*pi/180;

  if type1==2, M=1/M; P=-P; end
  alpha=(M*cos(P)-1)/(M*(M-cos(P)));
  tau=(M-cos(P))/sin(P)/omegad;
  if type1==1
    pr=-1/tau;
    zr=-1/(alpha*tau);
    kr=alpha;
  else
    zr=-1/tau;
    pr=-1/(alpha*tau);
    kr=1/alpha;
  end

  stkzr=[stkzr;zr];
  stkpr=[stkpr;pr];
  stkkr=[stkkr;kr];
  stkalpha=[stkalpha;alpha];
  stktau=[stktau;tau];
  p1=[p;pr];
  z1=[z;zr];
  k1=k*kr;
  [gain1,phi1,omega1]=frespc(z1,p1,k1,3.5,omega(1),omega(lomega),'k');

  subplot(2,1,1), %hold on
  semilogx(omega1,20*log10(gain1),'linewidth',1,'linestyle','--','color',colc)
  hold off

  subplot(2,1,2), %hold on
  semilogx(omega1,phi1,'linewidth',1,'linestyle','--','color',colc)
  hold off

  pause

  nfigs1=int2str(nfig1);
  figure(nfig1)

  [num,den]=dezpk(z1,p1,k1); dens=sumpol(num,den); [z2,p2,k2]=tozpk(num,dens);
  p3=[p2;0]; 

  % Step response

  if nnrip~=1, axis(TT), else, ordf(7), end
  col1=repco(nnrip);
  [y,t]=trespc(z2,p3,k2,col1); grid on
  if nnrip==1, TT=axis; end
  ytick=get(gca,'YTick'); divy=ytick(2)-ytick(1);
  ymax=max(y); ymin=min(y);
  while ymax>=TT(4)+divy, TT(4)=TT(4)+divy; end
  while ymin<=TT(3), TT(3)=TT(3)-divy; end
  TRF1=[gi,repco(1)]; TRF2=[gj,repco(2)];
  ABIL=1;
  if nnrip==1
    set(gcf,'Color',colbg)
    men_opt1=uimenu(gcf,'Label','Recall');
    if type1==1, mess='lag compensator'; else, mess='lead compensator'; end
    uimenu(men_opt1,'Label',mess);
    mess=[gi,', plant'];
    uimenu(men_opt1,'Label',mess,'Separator','on','Callback','tfmaux(1,.1)');
    mess=[gj,', compensator'];
    men_optt1=uimenu(men_opt1,'Label',mess);
    uimenu(men_optt1,'Label','zero-pole form','Callback','tfmaux(2,.1)');
    uimenu(men_optt1,'Label','time constant form','Callback','tfmaux(2,.1,1,1)');
    bdx=uimenu(gcf,'Label','Time axis');
    uimenu(bdx,'Label','enlarge x 2','Callback','ccregaux(11)');
    uimenu(bdx,'Label','enlarge x 5','Separator','on','Callback','ccregaux(12)');
    uimenu(bdx,'Label','enlarge x 10','Separator','on','Callback','ccregaux(13)');
    uimenu(bdx,'Label','recover the initial axis','Separator','on','Callback','ccregaux(14)');
    uimenu(gcf,'Label','Grid on/off','Callback','grid');
    uimenu(gcf,'Label','Zoom on/off','Callback','gridmng(7)')
    title('the closed-loop step response')
    hold on
    VFULL=get(gcf,'Position'); PO=get(gca,'Position');
    SC=diag([1/VFULL(3);1/VFULL(4);1/VFULL(3);1/VFULL(4)]);
    set(0,'DefaultUIControlUnits','normalized')
    Pos=[10 5 450 15]*SC;
    pbs2=uicontrol(gcf,...
    'Style','text',...
    'Position',Pos,...
    'HorizontalAlignment','left',...
    'BackgroundColor',colbg,...
    'ForegroundColor',vcol2(9,:),...
    'Visible','on');
    mess=['colors of subsequent plots: ',repco(1),', ',repco(2),', ',...
    repco(3),', ',repco(4),', ',repco(5),', ',repco(6),', ',repco(7)];
    text(.12,.04,mess,'FontWeight',wfont,'Units','normalized')
    zoom off, zoom out
  end % if nnrip

  set(pbs2,'String','press again the return key for another choice, enter 2 to exit');

  bx=input(['   press again the return key for another choice, enter 2 to exit : ']);
  if isempty(bx), bx=1; end

  if (bx==2)|(nnrip==7), xyz=1; end
  nnrip=nnrip+1; 

  figure(nfig)
  mess=sprintf('choose again with mouse + return, return key or button 2 to exit');
  set(pbs(19),'String',mess)

  end % if b==1
end % while xyz	

nnrip=nnrip-1; 
if nnrip==0, return, end

if nnrip>1
  figure(nfig1)
  mess=sprintf('select function by entering color ');
  set(pbs2,'String',mess)
  xx=0;
  while xx==0
    col=input('   select function by entering color : ','s');
    if col=='0', return, end
    ropc=0; for kk=1:7, ropc=ropc|strcmp(col,repco(kk)); end
    if ropc
      if find(repco==col)<=nnrip
      xx=1;
      end
    end
  end
else
  col='g';
end

eval('delete(nfig1)','err=1;')
clc, disp(' '), HALEG=HALEG(1:2);

q=find(repco==col);
alpha=stkalpha(q);
tau=stktau(q);
zr=stkzr(q);
pr=stkpr(q);
kr=stkkr(q);
type1=stktype1(q);

disp(' ')
if type1==1, insert='LAG'; else, insert='LEAD'; end
disp(['   THE OBTAINED ',insert,' COMPENSATOR :']), disp(' ')
fprintf('   alpha = %.4g , tau = %.4g sec\n',alpha,tau)
aa=importf(zr,pr,kr,gj);
if aa==1, tfg(gj), end

for kk=17:20, eval('delete(pbs(kk))','err=1;'), end
%for kk=17:20, delete(pbs(kk)), end
set(nfig,'Color',framesc), set(0,'DefaultUIControlUnits','pixel'), ordf(1)

% --- last line of compbode ---
