function convert(arg1,arg2,arg3,arg4)
%CONVERT  Conversion from continuous to discrete-time (TFI).
%   CONVERT,gi,gj - from TFI [CONVERT('gi,'gj') - from Matlab] converts
%   the continuous-time transfer function gi to the discrete-time
%   transfer function gj. Available options:
%      1 - Z-transform of the sampled inverse L-transform
%      2 - zero-order hold equivalence
%      3 - first-order hold equivalence
%   In all cases a finite delay can be interactively added.
%   If CONVERT is called without arguments, transfer functions gi and gj
%   can be specified in interactive mode.

% TFI Environment (A.Civolani & G.Marro) - 1994-96.

if nargin==0
 gi=input('   enter transfer function gi (input) : ','s');
 if isempty(gi), return, end
 if gi=='0', return, end
 if ~exist([gi,'.mat'])
  disp([' **** error: ',gi,'.mat does not exist']), return
 end
 gj=input('   enter transfer function gj (output) : ','s');
 if isempty(gj), return, end
 if gj=='0', return, end
elseif nargin>=2
 if ~exist([arg1,'.mat'])
  disp([' **** error: ',arg1,'.mat does not exist']), return
 else
  gi=arg1; gj=arg2;
 end
else
 disp(' **** error: not enough input arguments'), return
end
[z,p,k,str]=exportf(gi,1); if isempty(k), return, end
if length(z)>length(p)
 disp([' **** error: m > n in ',gi]), return
end
if str=='s'
 flg=0;
 while (flg~=1)&(flg~=2)&(flg~=3)
  disp(' ')
  disp('   1 - Z-transform of the sampled inverse L-transform')
  disp('   2 - zero-order hold equivalence')
  disp('   3 - first-order hold equivalence')
  flg=input('   your choice : ');
  if flg==0, return, end
 end
 flg1=1;
 while flg1==1
  t0=input('   enter a possible delay - default is zero : '); flg1=0;
  if length(t0)==0
   t0=0;
  elseif t0<0
   disp(' **** error: negative delay is not allowed'), flg1=1;
  end
 end
 T=lsamp;
 fprintf('\n   conversion from continuous-time to discrete-time;\n')
 fprintf('   the assumed sampling time is %.4g sec\n',T)
 fprintf('   the assumed delay is %.4g',t0)
 if t0>0, fprintf(' sec\n'), else, fprintf('\n'), end
 if (length(z)==length(p))&(flg==1)
  disp([' **** error: m = n in ',gi,' with option 1']), return
 end
 [zd,pd,kd]=cvtxz(z,p,k,T,t0,flg);
 aa=importf(zd,pd,kd,gj,1);
 if aa==1, tfg(gj), end
else
 disp(' ')
 disp(' **** warning: discrete-time system; conversion is not necessary')
 tfg(gi)
end
% --- last line of convert ---
