function deftf(arg1,arg2,arg3,arg4)
%DEFTF	  Definition of special transfer functions (TFI).
%   DEFTF,gi - from TFI [DEFTF('gi') from Matlab] provides one of the
%   following transfer functions:
%      1.  With any zero-pole map (introduced with the mouse)
%      2.  Bessel filter
%      3.  Butterworth filter
%      4.  Pade' delay [expansion of exp(-to*s)]
%   Type and parameters are chosen in interactive mode.
%   In cases 2 and 3 a suitable zero can be interactively added to
%   derive a reference model for a type 2 regulation loop.
%   If DEFTF is called without argument, transfer function gi can be
%   specified in interactive mode.

% TFI Environment (A.Civolani & G.Marro) - 1994-96.

if nargin==0
 gi=input('   enter transfer function : ','s');
 if isempty(gi), return, end
 if gi=='0', return, end
else
 gi=arg1;
end
flg1=0; clc
while flg1==0
 fprintf('\n   1) Define a zero-pole map with mouse')
 fprintf('\n   2) Bessel filter\n   3) Butterworth filter\n')
 fprintf(['   4) Pad',setstr(233),' expansion of finite delay\n'])
 flg=input('   enter your choice (press return to exit) : ');
 if (flg==0)|(length(flg)==0), return, end
 if ((flg==1)|(flg==2)|(flg==3)|(flg==4)), flg1=1; end
end
menupointer=[4,1,2,3]; flg=menupointer(flg);
if flg==4
 call=['deftfmou(''',gi,''')'];
 eval(call), return
end
flg1=0;
if flg==1 | flg==2
 while flg1==0
  n=input('   order of denominator : ');
  if length(n)~=0 & n>0, flg1=1; end
 end
 while flg1==1
  on=input('   corner frequency : ');
  if length(on)~=0 & on>0, flg1=2; end
 end
 if flg==1
  den=fbessel(on,n);
 else
  den=fbutter(on,n);
 end
 num=1; den(length(den))=1;
 [z1,p1,k1]=tozpk(num,den); p1=[p1;0];
 figure, ordf(2), nfig=gcf; nfigs=int2str(nfig);
 uimenu(gcf,'Label','Grid on/off','Callback','grid');
 [y,t]=trespc(z1,p1,k1,'g'); MM=max(y); TT=axis; TT(3)=0; TT(4)=1;
 while MM>=TT(4)
  TT(4)=TT(4)+.2;
 end
 axis(TT), ytick=0:.2:TT(4); set(gca,'Ytick',ytick), drawnow
 title(['the step response of ',gi])
 xyx=input(['   **** figure ',nfigs,' **** '],'s');
 flg2=input('   do you want a reference model for a type 2 system ? (1) : ');
 if flg2==1
  num1=num; den1=den;
  flg3=1;
  while flg3==1
   disp(' ')
   disp('   the order of both numerator and denominator is increased by 1')
   flg1=0;
   while flg1==0
    fprintf('   parameter alpha trades overshoot for settling time')
    alpha=input('   enter the value of alpha (from 1 to 100) : ');
    if (alpha>=1)&(alpha<=100), flg1=1; end
   end
   num=[(den1(n)+alpha/on),1];
   den=conv(den1,[alpha/on,1]);
   [z1,p1,k1]=tozpk(num,den); p1=[p1;0];
   figure(nfig), hold on
   [y,t]=trespc(z1,p1,k1,'c'); MM=max(y);
   while MM>=TT(4)
    TT(4)=TT(4)+.2; ytick=[ytick,TT(4)];
   end
   axis(TT), set(gca,'Ytick',ytick), drawnow, hold off
   xyx=input(['   **** figure ',nfigs,' **** '],'s');
   flg3=input('   do you want to change the value of alpha ? (1) : ');
   if flg3==0, return, end
  end
 end
else
 while flg1==0
  q=input('   order of denominator : ');
  if length(q)~=0 & q>0, flg1=1; end
 end
 while flg1==1
  p=input('   order of numerator (default same as denominator) : ');
  if length(p)==0, p=q; end
  if p>=0, flg1=2; end
 end
 while flg1==2
  to=input('   delay (sec) : ');
  if length(to)~=0 & to>0, flg1=3; end
 end
 [num,den]=fpade(to,p,q);
 [z1,p1,k1]=tozpk(num,den); p1=[p1;0];
 figure, ordf(2), nfig=gcf; nfigs=int2str(nfig);
 uimenu(gcf,'Label','Grid on/off','Callback','grid');
 [y,t]=trespc(z1,p1,k1,'g'); TT=axis;
 axis(TT), drawnow
 title(['the step response of ',gi])
 xyx=input(['   **** figure ',nfigs,' **** '],'s');
end
delete(nfig)
num=num/den(1); Denc=den/den(1); kc=num(1); Numc=num/kc;
if flg2==1, Denc(n+1)=kc*Numc(1); Denc(n+2)=kc*Numc(2); end
aa=stf(Numc,Denc,kc,'s',gi);
if aa==1, tfg(gi), end
% --- last line of deftf ---
