function dpopov(arg1,arg2,arg3,arg4)
%DPOPOV   Computation and plotting of the Popov diagram (TFI).
%   DPOPOV,gi - from TFI [DPOPOV('gi') - from Matlab] provides in the same
%   figure the Nyquist and Popov plots of gi(j*omega).

% TFI Environment (A.Civolani & G.Marro) - 1994-96.

global ABIL TRF1 gr nfig nfig1 HALEG FIGLEG LABLEG SUBPLEG

if nargin>0
 if ~exist([arg1,'.mat'])
  disp([' **** error: ',arg1,'.mat does not exist']), return
 else
  gi=arg1;
  [z,p,k,str]=exportf(gi,1); if isempty(k), return, end
  if str=='z'
   disp('   **** program popov only applies to continuous-time systems')
   return
  end
 end
end
%
aA=setstr(224); eA=setstr(232); oA=setstr(242); EA=setstr(200);
clf, nfig=gcf; nfigs=int2str(nfig); nargs=nargin; j=sqrt(-1); ABIL=0; hha1=[];
[vcol1,vcol2,repco]=coltbl; FIGLO=resfiglo;
FIGLEG=0; HALEG=[]; LABLEG=[]; SUBPLEG=0;
flx=0; fly=0; gr=0; ffp=1; ffq=1; to=0; flguic=0;
ms1='\n **** error: you cannot change figure while running\n';
ms2='   **** press any key to continue';
if exist('legen#.mat'), load legen#, else, legen=0; islegen=0; end
%
figure(nfig), clf, ordf(3)
if any(FIGLO(6,:))
 erx=0; eval('set(nfig,''Position'',FIGLO(6,1:4))','erx=1;')
end
men_opt=uimenu(nfig,'Label','Recall');
men_gr=uimenu(nfig,'Label','Grid on/off','Callback','gridmng(1)');
men_zoom=uimenu(nfig,'Label','Zoom on/off','Callback','gridmng(7)');
mess=['funzione di trasferimento : ',gi];
TRF1=[gi,repco(1)];
uimenu(men_opt,'Label',mess,'Callback','tfmaux(1,.76)');
to=0;
flg=0;
while flg==0
 if ffp==1
  s1=get(gca,'XLimMode'); isauto=strcmp(s1,'auto'); VV=axis;
  if isauto, VV=[-4,4,-3,3]; axis(VV), end
  [mag,phi,w1]=eval('frespc(z,p,k,5,repco(1),to)'); hold on
  y=mag.*exp(j*phi*pi/180); x2=real(y); y2=w1'.*imag(y);
  plot(x2,y2,'Color',vcol1(3,:),'LineStyle','-')
  if flx==1, set(gca,'xtick',xtick), end
  if fly==1, set(gca,'ytick',ytick), end
  flx=0; fly=0;
  if gr==1, grid on, end
  if ~isauto, axis(VV), end
  title(['Nyquist and Popov plots of ',gi])
  for kk=1:length(hha1), eval('delete(hha1(kk))','err=1;'), end, hha1=[];
  x3n=mag.*cos(phi*pi/180); y3n=mag.*sin(phi*pi/180); omg=w1;
  ncol=1; mg=mag;
  cfsize=get(gca,'FontSize'); fsize=8; lw=length(omg);
  rx=(VV(2)-VV(1))/(2*25); ry=(VV(4)-VV(3))/(1.5*25);
  ii=find((VV(1)<x3n)&(x3n<VV(2))&(VV(3)<y3n)&(y3n<VV(4))&(omg'>0));
  if length(ii)~=0
   hold on
   om1=omg(ii(1)); om2=omg(ii(length(ii)));
   omm=10^floor(log10(om1)); omi=omm;
   ve=[1.2,1.5,1.8,2.2,2.7,3.3,3.9,4.7,5.6,6.8,8.2,10];
   hhr=1; xv=x3n(1); yv=y3n(1); x=xv; y=yv;
   rrp=mg(2)>=mg(1);
   while (omm < om2)
    mx=1; my=1; [zz,kk]=min(abs(omg-omm));
    kk1=kk-1; kk2=kk+1; if kk==1, kk1=1; end, if kk==lw, kk2=lw; end
    ww1=omg(kk1); ww2=omg(kk2); ww=linspace(ww1,ww2,100);
    xx1=x3n(kk1); xx2=x3n(kk2); yy1=y3n(kk1); yy2=y3n(kk2);
    [zz,kh]=min(abs(ww-omm));
    if kh==1, kh=2; end
    x=xx1+(xx2-xx1)*kh/100; y=yy1+(yy2-yy1)*kh/100;
    x0=xx1+(xx2-xx1)*(kh-1)/100; y0=yy1+(yy2-yy1)*(kh-1)/100;
    stri=sprintf(' %.3g',omm);
    rop=(abs(x-xv)>length(stri)*(fsize/cfsize)*.8*rx);
    rop=rop|(abs(y-yv)>(fsize/cfsize)*1.2*ry);
    rop=rop & (omm > om1) & ((abs(x)>rx/2)|(abs(y)>ry/2)|rrp);
    rop=rop & (x>VV(1)) & (x<VV(2)) & (y>VV(3)) & (y<VV(4));
    if rop
     [han1,han2]=arrowl(x0,y0,x,y,rx/2,ry/2,repco(ncol),1);
     hha1=[hha1;han1]; hha1=[hha1;han2];
     xo=x+.5*(fsize/cfsize)*rx; yo=y-.5*(fsize/cfsize)*ry;
     if (x+length(stri)*(fsize/cfsize)*.75*rx)>VV(2)
      xo=x-length(stri)*(fsize/cfsize)*.75*rx; % yo=y+ry;
     end
     nnn=text(xo,yo,stri,'FontSize',fsize); hha1=[hha1;nnn];
     xv=x; yv=y;
    end
    if hhr==length(ve)
     omi=10*omi; omm=omi; hhr=1;
    else
     omm=ve(hhr)*omi; hhr=hhr+1;
    end
   end
  end  % length(ii)
  figure(nfig), hold off
 end % if ffp==1
 ABIL=1;
 if ffq==1, xyx=input(['   **** figure ',nfigs,' **** '],'s'); end
 clc, disp(' '), ffp=1; ffq=1;
 disp('   MENU :')
 disp(' ')
 disp('   1 - change the reference axes and plot again')
 if gr==0
  disp('   2 - grid on')
 else
  disp('   2 - grid off')
 end
 disp('   3 - recover the figure')
 zoom off, zoom out
 izy=input('   enter your choice (press return to exit) : ');
 if gcf~=nfig, fprintf(ms1), return, end
 if length(izy)==0
  cc=get(0,'Children');
  if any(cc==nfig)
   FIGLO(6,1:4)=get(nfig,'Position');
   save figlo# FIGLO
  else
   disp(' '), disp('   THE FIGURE HAS BEEN DELETED!')
  end
  eval('delete(nfig1)','err=1;')
  eval('delete(men_zoom)','err=1;'), eval('delete(men_gr)','err=1;')
  ordf(1), return
 end
 izy=round(izy);
 if (izy<1)|(izy>3), izy=3; end
 if izy==1
  clc, VV=axis;
  xtick=get(gca,'xtick'); ytick=get(gca,'ytick');
  divx=xtick(2)-xtick(1); divy=ytick(2)-ytick(1);
  ndiv=round([(VV(2)-VV(1))/divx,(VV(4)-VV(3))/divy]);
  fprintf('\n   the x axis interval is [%.4g %.4g];',VV(1),VV(2))
  indin=0; while indin==0
  TA1=input('   press return to maintain or enter new values: [xm xM] = ');
  ll1=length(TA1); if (ll1==0)|(ll1==2), indin=1; end, end, TA1=sort(TA1);
  fprintf('   the number of divisions is %.4g;',ndiv(1))
  if ll1==2
   fprintf('\n   press return to plot again with automatic scaling')
  else
   fprintf('\n   press return to plot again with the previous scale')
  end
  indin=0; while indin==0
  ndivt1=input('   or enter a new value: ndx = ');
  if (length(ndivt1)==0)|(length(ndivt1)==1), indin=1; end, end
  fprintf('\n   the y axis interval is [%.4g %.4g];',VV(3),VV(4))
  indin=0; while indin==0
  TA2=input('   press return to maintain or enter new values: [ym yM] = ');
  ll2=length(TA2); if (ll2==0)|(ll2==2), indin=1; end, end, TA2=sort(TA2);
  fprintf('   the number of divisions is %.4g;',ndiv(2))
  if ll2==2
   fprintf('\n   press return to plot again with automatic scaling')
  else
   fprintf('\n   press return to plot again with the previous scale')
  end
  indin=0; while indin==0
  ndivt2=input('   or enter a new value: ndy = ');
  if (length(ndivt2)==0)|(length(ndivt2)==1), indin=1; end, end
  if ll1==2, VV(1)=TA1(1); VV(2)=TA1(2); end
  if ll2==2, VV(3)=TA2(1); VV(4)=TA2(2); end
  if gcf~=nfig, fprintf(ms1), return, end
  rop=(ll1==2)|(ll2==2);
  if rop, axis(VV), end
  if ll1==0, flx=1; end
  if ll2==0, fly=1; end
  if length(ndivt1)~=0
   ndivt1=round(abs(ndivt1)); stepx=(VV(2)-VV(1))/ndivt1;
   xtick=VV(1):stepx:VV(2); flx=1;
  end
  if length(ndivt2)~=0
   ndivt2=round(abs(ndivt2)); stepy=(VV(4)-VV(3))/ndivt2;
   ytick=VV(3):stepy:VV(4); fly=1;
  end
  ffp=1; ffq=1;
 elseif izy==2
  if gr==0
   gr=1; grid on
  else
   gr=0; grid off
  end
  figure(nfig), ffp=0;
 elseif izy==3
  figure(nfig), ffp=0;
 end
 if gcf~=nfig, fprintf(ms1), return, end
end
% --- last line of dpopov ---
