function gpmarg(arg1,arg2,arg3,arg4)
%GPMARG   Gain and phase margins (TFI).
%   GPMARG,gi - from TFI [GPMARG('gi') - from Matlab] displays gain and
%   phase margin of gi.
%   GPMARG,gi,1 - from TFI [GPMARG('gi','1') - from Matlab] provides
%   options in interactive mode for computation of gain margin at phase
%   different from -180 and phase margin at gain different from 1.
%   If GPMARG is called without argument, transfer function gi can be
%   specified in interactive mode.

% TFI Environment (A.Civolani & G.Marro) - 1994-96.

nargs=nargin;
if nargs==0
 gi=input('   enter transfer function : ','s');
 if isempty(gi), return, end
 if gi=='0', return, end
 if ~exist([gi,'.mat'])
  disp([' **** error: ',gi,'.mat does not exist']), return
 end
else
 if ~exist([arg1,'.mat'])
  disp([' **** error: ',arg1,'.mat does not exist']), return
 else
  gi=arg1;
 end
end
[z,p,k,str]=exportf(gi,1); if isempty(k), return, end
if strcmp(str,'z')
 T=lsamp; [z,p,k]=diwplane(z,p,k,T);
end
[num,den]=dezpk(z,p,k);
%
xyx=0;
if (nargs==0)|(nargs==2)
 xyx=1;
end
if xyx==1
 phi0=input('   enter the reference phase for gain margin (default -180) : ');
 if length(phi0)==0, phi0=-180; end
 mag0=-1;
 while mag0<=0
 mag0=input('   enter the reference gain for phase margin (default 1)    : ');
 if length(mag0)==0, mag0=1; end
 end
else
 phi0=-180; mag0=1;
end
[gm,pm,omg,omp]=gpcross1(z,p,k,mag0,phi0);
%
%   gain margin
%
if isnan(omg)
 fprintf('\n   gain margin non-computable\n')
else
 gm=1/gm;
 if strcmp(str,'z')
  omg=(1/T)*(angle(1+j*omg*T/2)-angle(1-j*omg*T/2));
 end
 fprintf(['\n   gain margin of transfer function ',gi,'(',str,')'])
 fprintf(': %.4g',gm)
 fprintf(' (%.4g dB)\n',20*log10(gm))
 fprintf('   (phase = %.4g degrees)',phi0)
 fprintf(' at frequency: %.4g rad/sec\n',omg)
end
%
%   phase margin
%
if isnan(omp)
 fprintf('\n   phase margin non-computable\n')
else
pm=180+pm;
if strcmp(str,'z')
 omp=(1/T)*(angle(1+j*omp*T/2)-angle(1-j*omp*T/2));
end
 fprintf(['\n   phase margin of transfer function ',gi,'(',str,')'])
 fprintf(': %.4g degrees\n',pm)
 fprintf('   (gain = %.4g)',mag0)
 fprintf(' at frequency: %.4g rad/sec\n',omp)
end
disp(' ')
% ----- last line of gpmarg -----
