function pidnich(arg1,arg2,arg3,arg4)
%PIDNICH  Design of PD, PI and PID regulators with the Nichols plot (TFI).
%    PIDNICH,gp,gc - from Transfer Function Interpreter
%    PIDNICH('gp','gc') - from Matlab
%    gp: transfer function of the plant (input)
%    gc: transfer function of the regulator (output)
%
%    If PIDNICH is called without arguments, transfer functions gp and gc
%    can be specified in interactive mode.

% TFI Environment (A.Civolani & G.Marro) - 1994-96.

global pbs gr MAG PHI OM VV repco gi gj NPKK...
 prfx prfy ABIL TRF1 TRF2 zpl ppl kpl zcomp pcomp kcomp TT Kc Ti Td...
 stkkc stktaud stktaui stktype NPKK vcol1 vcol2 ax had hha1 car car1...
 HALEG FIGLEG LABLEG SUBPLEG

nfig=gcf; nfigs=int2str(nfig); ABIL=1; car=0; car1=[]; had=[]; gr=0;
zcomp=[]; pcomp=[]; kcomp=1; NPKK=0; pbs=zeros(1,100); izz=0;
FIGLO=resfiglo; xxtt=0; pbs(54)=xxtt;
FIGLEG=nfig; HALEG=[]; LABLEG=[]; SUBPLEG=0;
if exist('legen#.mat'), load legen#, else, legen=0; islegen=0; end
pbs(58)=legen; pbs(59)=islegen;
if nargin==0
 arg1='xx'; flg=0;
 while flg==0
  mess='   enter the transfer function of the plant (input) : ';
  gi=input(mess,'s');
  if isempty(gi), gi='0'; end, if gi=='0', return, end
  if ~isempty(gi)
   if ~exist([gi,'.mat'])
    disp([' **** error: ',gi,'.mat does not exist'])
   else
    flg=1;
   end
  end
 end
 while flg==1
  mess='   enter the transfer function of the regulator (output) : ';
  gj=input(mess,'s');
  if isempty(gj), gj='0'; end, if gj=='0', return, end
  if ~isempty(gj), flg=2; end
 end
elseif nargin>=2
 if ~exist([arg1,'.mat'])
  disp([' **** error: ',arg1,'.mat does not exist']), return
 else
  gi=arg1; gj=arg2;
 end
else
 disp(' **** error: not enough input arguments'), return
end
[zdata,pdata,kdata,str1]=exportf(gi,1); if isempty(kdata), return, end
zpl=zdata; ppl=pdata; kpl=kdata;
if str1=='z'
 fprintf('\n   this program does not work for discrete-time systems:')
 fprintf(' use pidd instead\n')
 return
end
figure(nfig)
tfblocks(3,gj,gi,'','','','','','','','')
%
flgll=0;
ll=input('   information on the design method ? (1) : ');
if isempty(ll), ll=0; end
if ll==1
 ccc='prninfo(1)';
 eval(ccc)
 flgin=0;
 while flgin==0
  mess='   enter 1 to continue with design, 0 to exit : '; xxx=input(mess);
  if xxx==1, flgin=1; elseif xxx==0, ordf(1), return, end
 end
end
%
clf, ordf(3), hold off
VFULL=get(gcf,'Position'); PO=get(gca,'Position');
SC=diag([1/VFULL(3);1/VFULL(4);1/VFULL(3);1/VFULL(4)]);
set(0,'DefaultUIControlUnits','normalized')
if any(FIGLO(4,:))
 erx=0; eval('set(nfig,''Position'',FIGLO(4,1:4))','erx=1;')
end
[vcol1,vcol2,repco,repcos]=coltbl; colbg=vcol1(8,:);
%
nfig1=nfig+1; figure(nfig1), clf
if any(FIGLO(4,:))
 erx=0; eval('set(nfig1,''Position'',FIGLO(4,5:8))','erx=1;')
 if erx==1, ordf(6), end
else
 ordf(6)
end
pbs(51)=uimenu(gcf,'Label','Recall');
men1=uimenu(gcf,'Label','Time axis'); pbs(92)=men1;
uimenu(men1,'Label','enlarge x 2',...
'Callback','pidniaux(25)');
uimenu(men1,'Label','enlarge x 5','Separator','on',...
'Callback','pidniaux(26)');
uimenu(men1,'Label','enlarge x 10','Separator','on',...
'Callback','pidniaux(27)');
uimenu(men1,'Label','enlarge x 20','Separator','on',...
'Callback','pidniaux(28)');
uimenu(men1,'Label','recover the initial axis','Separator','on',...
'Callback','pidniaux(29)');
pbs(93)=uimenu(gcf,'Label','Grid on/off','Callback','gridmng(13)');
pbs(94)=uimenu(gcf,'Label','Zoom on/off','Callback','gridmng(7)');
set(pbs(51),'Enable','off')
for kk=92:94, set(pbs(kk),'Enable','off'), end
BB=VFULL(4);
Pos=[150 BB-120 200 20]*SC;
pbs(47)=uicontrol(gcf,...
'Style','Text',...
'Position',Pos,...
'BackgroundColor',colbg,...
'ForegroundColor',vcol2(2,:),...
'Visible','off',...
'String','WAIT, PLEASE');
Pos=[20 5 50 20]*SC;
pbs(48)=uicontrol(gcf,...
'Style','push',...
'Position',Pos,...
'Visible','off',...
'String','Menu',...
'Callback','pidniaux(39)');
%
nfig2=nfig+2; figure(nfig2), clf
if any(FIGLO(4,:))
 erx=0; eval('set(nfig2,''Position'',FIGLO(4,9:12))','erx=1;')
 if erx==1, ordf(7), end
else
 ordf(7)
end
pbs(52)=uimenu(gcf,'Label','Recall');
men2=uimenu(gcf,'Label','Frequency axis'); pbs(97)=men2;
men2_1=uimenu(men2,'Label','Left');
uimenu(men2_1,'Label','plus one decade','Callback','pidniaux(45)')
uimenu(men2_1,'Label','minus one decade','Callback','pidniaux(46)')
men2_2=uimenu(men2,'Label','Right','Separator','on');
uimenu(men2_2,'Label','plus one decade','Callback','pidniaux(47)')
uimenu(men2_2,'Label','minus one decade','Callback','pidniaux(48)')
pbs(98)=uimenu(gcf,'Label','Grid on/off','Callback','gridmng(14)');
pbs(99)=uimenu(gcf,'Label','Zoom on/off','Callback','gridmng(8)');
set(pbs(52),'Enable','off')
for kk=97:99, set(pbs(kk),'Enable','off'), end
BB=VFULL(4);
Pos=[150 BB-120 200 20]*SC;
pbs(49)=uicontrol(gcf,...
'Style','Text',...
'Position',Pos,...
'BackgroundColor',colbg,...
'ForegroundColor',vcol2(2,:),...
'Visible','off',...
'String','WAIT, PLEASE');
Pos=[20 5 50 20]*SC;
pbs(50)=uicontrol(gcf,...
'Style','push',...
'Position',Pos,...
'Visible','off',...
'String','Menu',...
'Callback','pidniaux(39)');
figure(nfig1), figure(nfig), set(gca,'Box','on')
%
aa=importf([],[],1,gj);
j=sqrt(-1); pbs(28)=0; pbs(29)=nfig; pbs(1)=0;
stkkc=zeros(1,6); stktaui=zeros(1,6); stktaud=zeros(1,6); stktype=zeros(1,6);
%
TRF1=[gi,repco(7)];
men_opt=uimenu(gcf,'Label','Recall');
mess='PD, PI or PID regulator design';
uimenu(men_opt,'Label',mess);
mess=[gi,', plant'];
uimenu(men_opt,'Label',mess,'Separator','on',...
'Callback','tfmaux(1,1,0)');
TRF2=[gj,repco(1)];
mess=[gj,', regulator'];
men_optt=uimenu(men_opt,'Label',mess);
uimenu(men_optt,'Label','zero-pole',...
'Callback','tfmaux(2,1,0)');
uimenu(men_optt,'Label','time-constant',...
'Callback','tfmaux(2,1,0,1)');
%
%  TOP MENU
%
pbs(2)=uimenu(gcf,'Label','Change axes');
mens1=uimenu(pbs(2),'Label','y top');
mens2=uimenu(pbs(2),'Label','y bottom');
mens3=uimenu(pbs(2),'Label','x left','Separator','on');
mens4=uimenu(pbs(2),'Label','x right');
pbs(3)=uimenu(gcf,'Label','Grid on/off','Callback','gridmng(1)');
%
uimenu(mens3,...
'Label','plus 30 degrees',...
'Callback','pidniaux(11)');
uimenu(mens3,...
'Label','minus 30 degrees',...
'Callback','pidniaux(12)');
uimenu(mens4,...
'Label','plus 30 degrees',...
'Callback','pidniaux(13)');
uimenu(mens4,...
'Label','minus 30 degrees',...
'Callback','pidniaux(14)');
%
uimenu(mens1,...
'Label','plus 20 dB',...
'Callback','pidniaux(15)');
uimenu(mens1,...
'Label','minus 20 dB',...
'Callback','pidniaux(16)');
uimenu(mens2,...
'Label','plus 20 dB',...
'Callback','pidniaux(17)');
uimenu(mens2,...
'Label','minus 20 dB',...
'Callback','pidniaux(18)');
%
uimenu(pbs(2),...
'Label','standard',...
'Separator','on',...
'Callback','pidniaux(19)');
uimenu(pbs(2),...
'Label','M and N loci on/off',...
'Separator','on',...
'Callback','pidniaux(20)');
%
BB=VFULL(4)-24;
%
for kk=1:10 % 31->40
 Pos=[4 BB-20*(kk-1) 114 20]*SC;
 pbs(30+kk)=uicontrol(gcf,...
 'Visible','off',...
 'Position',Pos);
end
set(pbs(39),'Position',[118 BB-120 114 20]*SC)
set(pbs(40),'Position',[118 BB-140 114 20]*SC)
Pos=[118 BB-160 114 20]*SC;
pbs(81)=uicontrol(gcf,'Visible','off','Position',Pos);
%
set(pbs(31),'Style','text','String','Functions:','Visible','on')
set(pbs(32),'Style','push','String','PD','CallBack','pidniaux(21)')
set(pbs(33),'Style','push','String','PI','CallBack','pidniaux(22)')
set(pbs(34),'Style','push','String','PID','CallBack','pidniaux(23)')
set(pbs(35),'Style','push','String','Continue','CallBack','pidniaux(2)')
set(pbs(36),'Style','push','String','Another regulator',...
'CallBack','pidniaux(1)')
set(pbs(37),'Style','push','String','Step response',...
'CallBack','pidniaux(31)')
set(pbs(38),'Style','push','String','Bode diagrams',...
'CallBack','pidniaux(32)')
%
wwdth=186;
lskip=round(VFULL(4)*30/480);
for kk=21:25
 Pos=[14 lskip+(5.5-kk+20)*15 wwdth 15]*SC;
 pbs(kk)=uicontrol(gcf,...
 'Style','text',...
 'Position',Pos,...
 'HorizontalAlignment','left',...
 'BackgroundColor',colbg,...
 'Visible','off');
end
Pos=[14 10 wwdth 15]*SC;
pbs(26)=uicontrol(gcf,...
'Style','text',...
'Position',Pos,...
'HorizontalAlignment','left',...
'BackgroundColor',colbg,...
'String','dotted plots: feedback-only DA',...
'ForegroundColor',vcol2(9,:),...
'Visible','off');
%
BB=VFULL(4)-14; llth=400;
%
Pos=[128 BB llth 15]*SC;
pbs(5)=uicontrol(gcf,...
'Style','text',...
'Position',Pos,...
'HorizontalAlignment','left',...
'BackgroundColor',colbg,...
'Visible','off');
%
Pos=[128 BB-15 llth 15]*SC;
pbs(6)=uicontrol(gcf,...
'Style','text',...
'Position',Pos,...
'HorizontalAlignment','left',...
'BackgroundColor',colbg,...
'Visible','off');
%
Pos=[124 BB-55 90 20]*SC;
pbs(19)=uicontrol(gcf,...
'Style','text',...
'Position',Pos,...
'HorizontalAlignment','left',...
'BackgroundColor',colbg,...
'Visible','off');
%
llth=110;
for kk=1:10 % 7->16
 Pos=[4 BB-(kk-1)*15 llth 15]*SC;
 pbs(6+kk)=uicontrol(gcf,...
 'Style','text',...
 'Position',Pos,...
 'HorizontalAlignment','left',...
 'BackgroundColor',colbg,...
 'Visible','off');
end
Pos=[14 BB-13*15 llth 15]*SC;
pbs(27)=uicontrol(gcf,...
'Style','text',...
'Position',Pos,...
'HorizontalAlignment','left',...
'BackgroundColor',colbg,...
'Visible','off');
Pos=[14 150 50 20]*SC;
pbs(20)=uicontrol(gcf,...
'Style','push',...
'Position',Pos,...
'HorizontalAlignment','left',...
'Visible','off',...
'String','Back');
Pos=[204 BB-120 200 15]*SC;
pbs(17)=uicontrol(gcf,...
'Style','text',...
'Position',Pos,...
'HorizontalAlignment','left',...
'BackgroundColor',colbg,...
'Visible','off',...
'String','SELECTION NOT ACCEPTABLE');
Pos=[10 5 180 15]*SC;
pbs(18)=uicontrol(gcf,...
'Style','text',...
'Position',Pos,...
'HorizontalAlignment','left',...
'BackgroundColor',colbg,...
'ForegroundColor',vcol2(9,:),...
'Visible','off',...
'String','PRESS RETURN TO EXIT');
BB=VFULL(4);
Pos=[20 5 50 20]*SC;
pbs(53)=uicontrol(gcf,...
'Style','push',...
'Position',Pos,...
'String','Exit',...
'Callback','pidniaux(40)');
%
%  computation of acos
%
prfx=linspace(0,pi/2,200); prfy=-20*log10(cos(prfx));
prfx=180/pi*prfx;
%
% definition of the axes
%
delete(gca)
ax=zeros(3,1);
ax(1)=axes('Position',[0.25,.32,.675,.595],'Box','on');
set(ax(1),'FontSize',10)
ax(2)=axes('Position',[0.37,.05,.24,.18],'Box','on');
set(ax(2),'FontSize',8)
title('plant output','FontSize',10);
ax(3)=axes('Position',[0.685,.05,.24,.18],'Box','on');
set(ax(3),'FontSize',8)
title('regulator output','FontSize',10)
axes(ax(1))
%
drawnow
pidniaux(1)
figure(nfig)
xyx=input(['   **** figure ',nfigs,' **** '],'s');
if islegen
 VV=axis; xx=2*VV(1)-VV(2); yy=2*VV(3)-VV(4);
 hold on, HAL=plot(xx,yy,xx,yy);
 set(HAL(1),'Color',vcol1(7,:)), set(HAL(2),'Color',vcol1(pbs(1),:))
 hold off
end
HALEG=HAL;
cc=get(0,'Children');
if (any(cc==nfig))&(any(cc==nfig1))&(any(cc==nfig2))
 FIGLO(4,1:4)=get(nfig,'Position');
 FIGLO(4,5:8)=get(nfig1,'Position');
 FIGLO(4,9:12)=get(nfig2,'Position');
 save figlo# FIGLO
else
 disp(' '), disp('   A FIGURE HAS BEEN DELETED!'), ordf, return
end
if stktype(1)==0
 eval('delete(nfig1)','err=1;'), eval('delete(nfig2)','err=1;')
 clf, ordf(1), return
end
flcomp=pbs(1);
if islegen
 ac=0;
 if legen==1
  ac=input('   do you want to clean the legend ? (1) : ');
  if length(ac)==0, ac=0; end
 end
 if ac==1
  makeleg(1)
  xyx=input(['   **** figure ',nfigs,' **** '],'s');
 end
 mess='   use "makeleg" to clean the legend after moving if necessary';
 if legen==1, disp(' '), disp(mess), end
end
clc
if stktype(flcomp)==0, flcomp=flcomp-1; end
if flcomp>1
 xx=0;
 while xx==0
  col=input('   select function by entering color: ','s');
  if col=='0', return, end
  ropc=0; for kk=1:6, ropc=ropc|strcmp(col,repco(kk)); end
  if ropc
   if find(repco==col)<=flcomp
    xx=1;
   end
  end
 end
else
 col=repco(1);
end
q=find(repco==col);
Kc=stkkc(q); Ti=stktaui(q); Td=stktaud(q); typ=stktype(q);
clc, disp(' ')
disp('   THE REGULATOR OBTAINED : ')
if typ==1
 fprintf('\n   Type PD : ')
 fprintf('Kc=%.4g, Td=%.4g sec\n',Kc,Td)
 if Td~=0
  kcomp=Kc*Td; zcomp=-1/Td; pcomp=[];
 else
  kcomp=Kc; zcomp=[]; pcomp=[];
 end
elseif typ==2
 fprintf('\n   Type PI : ')
 kcomp=Kc; zcomp=-1/Ti; pcomp=0;
 if ~finite(Ti),
  fprintf('Kc=%.4g, Ti=%.4g \n',Kc,Ti), zcomp=[]; pcomp=[];
 else
  fprintf('Kc=%.4g, Ti=%.4g sec\n',Kc,Ti)
 end
elseif typ==3
 fprintf('\n   Type PID : ')
 fprintf('Kc=%.4g, Ti=%.4g sec',Kc,Ti), fprintf(', Td=%.4g sec\n',Td)
 kcomp=Kc*Td; zcomp=roots([1,1/Td,1/(Ti*Td)]); pcomp=0;
end
aa=importf(zcomp,pcomp,kcomp,gj);
if aa==1, tfg(gj), end
eval('delete(ax(2))','err=1;'), eval('delete(ax(3))','err=1;')
eval('delete(nfig1)','err=1;'), eval('delete(nfig2)','err=1;'), ABIL=0;
set(gcf,'Visible','off'), drawnow
eval('delete(pbs(2))','err=1;'), eval('delete(pbs(3))','err=1;')
for kk=5:27, eval('delete(pbs(kk))','err=1;'), end
for kk=31:40, eval('delete(pbs(kk))','err=1;'), end
eval('delete(pbs(45))','err=1;'), eval('delete(pbs(53))','err=1;')
eval('delete(pbs(81))','err=1;')
set(ax(1),'Position',PO)
set(0,'DefaultUIControlUnits','pixel'), ordf(1), set(gcf,'Visible','on')
if stktype(q)~=2
 flgin=0;
 while flgin==0
  mess='   feedback-only derivative action ? (1) : '; xxx=input(mess);
  if isempty(xxx), return, end
  if xxx~=1, return, end
  if xxx==1
   mess='   enter a name for the feedforward transfer function : ';
   ggd=input(mess,'s'); if ggd=='0', return, end
   mess='   enter a name for the feedback transfer function    : ';
   ggf=input(mess,'s'); if ggf=='0', return, end
   if isstr(ggd)&isstr(ggf), flgin=1; end
  end
 end
 if isstr(arg1)
  mess=[ggd,', feedforward unit'];
  uimenu(men_opt,'Label',mess);
  mess=[ggf,', feedback unit'];
  uimenu(men_opt,'Label',mess);
 end
 figure, tfblocks(4,ggd,gi,ggf,'','','','','','',''), delete(gcf)
 if stktype(q)==1
  numff=Kc; denff=1; numfb=[Td 1]; denfb=1;
 else
  numff=Kc*[Ti 1]; denff=[Ti 0]; numfb=[Ti*Td Ti 1]; denfb=[Ti 1];
 end
 [zz,pp,kk]=tozpk(numff,denff);
 zz=round(zz*10^14)/10^14;
 aa=importf(zz,pp,kk,ggd);
 if aa==1, tfg(ggd), end
 [zz,pp,kk]=tozpk(numfb,denfb);
 pp=round(pp*10^14)/10^14;
 aa=importf(zz,pp,kk,ggf);
 if aa==1, tfg(ggf), end
end
% --- last line of pidnich ---
