function routh(arg1,arg2,arg3,arg4)
%ROUTH	  Stability intervals with the Routh criterion (TFI).
%   ROUTH,gi - from TFI [ROUTH('gi') - from Matlab] provides the stability
%   intervals in terms of the loop gain K for the continuous or
%   discrete-time feedback system with transfer function gi.
%   The corresponding characteristic equation is:
%
%	    1 + K gi(s) = 0   or   1 + K gi(z) = 0
%
%   In the discrete-time case stability is referred to the unit circle.
%   If ROUTH is called without argument, transfer function gi can be
%   specified in interactive mode.

% TFI Environment (A.Civolani & G.Marro) - 1994-96.

if nargin==0
 gi=input('   enter transfer function : ','s');
 if isempty(gi), return, end
 if ~exist([gi,'.mat'])
  disp([' **** error: ',gi,'.mat does not exist'])
 end
else
 if ~exist([arg1,'.mat'])
  disp([' **** error: ',arg1,'.mat does not exist']), return
 else
  gi=arg1;
 end
end
[z,p,k,str]=exportf(gi,1); if isempty(k), return, end
if max(length(p),length(z))==0
 disp(' '), disp([' **** error: no dynamics in ',gi]), return
end
figure, nnf=gcf; tfblocks(3,'K',gi,'','','','','','','','')
err=0; eval('delete(nnf)','err=1;')
disp(' ')
fprintf('   characteristic equation:')
if strcmp(str,'s')
 disp(['   1 + K ',gi,'(s) = 0 ;'])
 routhc(z,p,k)
else
 disp(['   1 + K ',gi,'(z) = 0 ;'])
 routhd(z,p,k,lsamp)
end
% --- last line of routh ---
