function wplane(arg1,arg2,arg3,arg4)
%WPLANE   Direct and inverse wplane transform (TFI).
%   WPLANE,gi,gj - from TFI [WPLANE('gi,'gj') - from Matlab] transforms
%   the discrete-time transfer function gi to the continuous-time gj
%   according to the wplane equivalence, or the continuous-time transfer
%   function gi to the discrete-time gj according to the inverse wplane
%   equivalence:
%
%           1 + wT/2          2 (z - 1)
%      z =  --------  ;  w =  ---------  .
%           1 - wT/2          T (z + 1)
%
%   If WPLANE is called without arguments, transfer functions gi and gj
%   can be specified in interactive mode.

% TFI Environment (A.Civolani & G.Marro) - 1994-96.

if nargin==0
 gi=input('   enter transfer function gi (input) : ','s');
 if isempty(gi), return, end
 if gi=='0', return, end
 if ~exist([gi,'.mat'])
  disp([' **** error: ',gi,'.mat does not exist']), return
 end
 gj=input('   enter transfer function gj (output) : ','s');
 if isempty(gj), return, end
 if gj=='0', return, end
elseif nargin>=2
 if ~exist([arg1,'.mat'])
  disp([' **** error: ',arg1,'.mat does not exist']), return
 else
  gi=arg1; gj=arg2;
 end
else
 disp(' **** error: not enough input arguments'), return
end
[z,p,k,str]=exportf(gi,1); if isempty(k), return, end
if length(z)>length(p)
 disp([' **** error: m > n in ',gi]), return
end
T=lsamp;
if strcmp(str,'s')
 fprintf('\n   transformation from continuous to discrete according to')
 fprintf('\n   the inverse wplane equivalence; the assumed sampling time is ')
 fprintf('%.4g sec\n',T)
 [zrd,prd,krd]=inwplane(z,p,k,T);
 aa=importf(zrd,prd,krd,gj,1);
else
 fprintf('\n   transformation from discrete to continuous according to')
 fprintf('\n   the direct wplane equivalence; the assumed sampling time is ')
 fprintf('%.4g sec\n',T)
 [zr,pr,kr]=diwplane(z,p,k,T);
 aa=importf(zr,pr,kr,gj);
end
if aa==1, tfg(gj), end
% --- last line of wplane ---
